@@ -1,22 +1,16 @@
 <?php
 /* vim: set expandtab tabstop=4 shiftwidth=4: */
-// +----------------------------------------------------------------------+
-// | PHP version 4                                                        |
-// +----------------------------------------------------------------------+
-// | Copyright (c) 1997-2002, 2008 The PHP Group                                |
-// +----------------------------------------------------------------------+
-// | This source file is subject to version 3.0 of the PHP license,       |
-// | that is bundled with this package in the file LICENSE, and is        |
-// | available at through the world-wide-web at                           |
-// | http://www.php.net/license/3_0.txt.                                  |
-// | If you did not receive a copy of the PHP license and are unable to   |
-// | obtain it through the world-wide-web, please send a note to          |
-// | license@php.net so we can mail you a copy immediately.               |
-// +----------------------------------------------------------------------+
-// | Authors: Ian Eure <ieure@php.net>                                    |
-// +----------------------------------------------------------------------+
-//
-// $Id: Type.php 273631 2009-01-16 11:49:45Z cweiske $
+/**
+ * Part of MIME_Type
+ *
+ * PHP version 4 and 5
+ *
+ * @category File
+ * @package  MIME_Type
+ * @author   Ian Eure <ieure@php.net>
+ * @license  http://www.gnu.org/copyleft/lesser.html LGPL
+ * @link     http://pear.php.net/package/MIME_Type
+ */
 
 require_once 'PEAR.php';
 
@@ -28,10 +22,12 @@ $_fileCmd = 'file';
  *
  * @category MIME
  * @package  MIME_Type
- * @license  PHP License 3.0
- * @version  @version@
- * @link     http://pear.php.net/package/MIME_Type
  * @author   Ian Eure <ieure@php.net>
+ * @license  http://www.gnu.org/copyleft/lesser.html LGPL
+ * @version  Release: @version@
+ * @link     http://pear.php.net/package/MIME_Type
+ *
+ * @method string autoDetect($file, $params = false) Autodetect a file's MIME-type (can be called statically and on an MIME_Type object)
  */
 class MIME_Type
 {
@@ -40,21 +36,21 @@ class MIME_Type
      *
      * @var string
      */
-    var $media = '';
+    public $media = '';
 
     /**
      * The MIME media sub-type
      *
      * @var string
      */
-    var $subType = '';
+    public $subType = '';
 
     /**
      * Optional MIME parameters
      *
      * @var array
      */
-    var $parameters = array();
+    public $parameters = array();
 
     /**
      * List of valid media types.
@@ -63,7 +59,7 @@ class MIME_Type
      *
      * @var array
      */
-    var $validMediaTypes = array(
+    public $validMediaTypes = array(
         'text',
         'image',
         'audio',
@@ -73,6 +69,42 @@ class MIME_Type
         'message'
     );
 
+    /**
+     * If the finfo functions shall be used when they are available
+     *
+     * @var boolean
+     */
+    public $useFinfo = true;
+
+    /**
+     * If mime_content_type shall be used when available
+     *
+     * @var boolean
+     */
+    public $useMimeContentType = true;
+
+    /**
+     * If the file command shall be used when available
+     *
+     * @var boolean
+     */
+    public $useFileCmd = true;
+
+    /**
+     * If the in-built file extension detection shall be used
+     *
+     * @var boolean
+     */
+    public $useExtension = true;
+
+    /**
+     * Path to the "magic" file database.
+     * If NULL, the default one is used
+     *
+     * @var string
+     */
+    public $magicFile = null;
+
 
     /**
      * Constructor.
@@ -82,36 +114,81 @@ class MIME_Type
      * This is useful, but not quite as useful as parsing a type.
      *
      * @param string $type MIME type
-     *
-     * @return void
      */
-    function MIME_Type($type = false)
+    public function __construct($type = false)
     {
         if ($type) {
             $this->parse($type);
         }
     }
 
+    /**
+     * Make "autoDetect" method available.
+     *
+     * @param string $method    Method to call. Only "autoDetect" supported.
+     * @param array  $arguments Function parameters.
+     *
+     * @return string MIME type
+     */
+    public function __call($method, $arguments)
+    {
+        if ($method !== 'autoDetect') {
+            return trigger_error(
+                'Call to undefined method MIME_TYPE::' . $method . '()',
+                E_USER_ERROR
+            );
+        }
+        $type = call_user_func_array(array($this, '_autoDetect'), $arguments);
+        $this->parse($type);
+        return $type;
+    }
+
+    /**
+     * Make "autoDetect" method available statically.
+     *
+     * @param string $method    Method to call. Only "autoDetect" supported.
+     * @param array  $arguments Function parameters.
+     *
+     * @return string MIME type
+     */
+    public static function __callStatic($method, $arguments)
+    {
+        if ($method !== 'autoDetect') {
+            return trigger_error(
+                'Call to undefined method MIME_TYPE::' . $method . '()',
+                E_USER_ERROR
+            );
+        }
+
+        $mt = new MIME_Type();
+        return call_user_func_array(array($mt, '_autoDetect'), $arguments);
+    }
 
     /**
      * Parse a mime-type and set the class variables.
      *
      * @param string $type MIME type to parse
      *
-     * @return void
+     * @return boolean True if the type has been parsed, false if not
      */
-    function parse($type)
+    public function parse($type)
     {
+        if ($type instanceof PEAR_Error) {
+            return false;
+        }
+
         $this->media      = $this->getMedia($type);
         $this->subType    = $this->getSubType($type);
         $this->parameters = array();
         if (MIME_Type::hasParameters($type)) {
-            require_once 'MIME/Type/Parameter.php';
+            include_once 'MIME/Type/Parameter.php';
             foreach (MIME_Type::getParameters($type) as $param) {
                 $param = new MIME_Type_Parameter($param);
                 $this->parameters[$param->name] = $param;
             }
         }
+
+        return true;
     }
 
 
@@ -121,9 +198,8 @@ class MIME_Type
      * @param string $type MIME type to check
      *
      * @return boolean true if $type has parameters, false otherwise
-     * @static
      */
-    function hasParameters($type)
+    public static function hasParameters($type)
     {
         if (strstr($type, ';')) {
             return true;
@@ -138,9 +214,8 @@ class MIME_Type
      * @param string $type MIME type to get parameters of
      *
      * @return array $type's parameters
-     * @static
      */
-    function getParameters($type)
+    public static function getParameters($type)
     {
         $params = array();
         $tmp    = explode(';', $type);
@@ -157,9 +232,8 @@ class MIME_Type
      * @param string $type MIME type string
      *
      * @return string MIME type with parameters removed
-     * @static
      */
-    function stripParameters($type)
+    public static function stripParameters($type)
     {
         if (strstr($type, ';')) {
             return substr($type, 0, strpos($type, ';'));
@@ -171,33 +245,33 @@ class MIME_Type
     /**
      * Removes comments from a media type, subtype or parameter.
      *
-     * @param string $string   String to strip comments from
-     * @param string &$comment Comment is stored in there.
+     * @param string $string  String to strip comments from
+     * @param string $comment Comment is stored in there.
+     *                        Do not set it to NULL if you want the comment.
      *
-     * @return string   String without comments
-     * @static
+     * @return string String without comments
      */
-    function stripComments($string, &$comment)
+    public static function stripComments($string, &$comment)
     {
         if (strpos($string, '(') === false) {
             return $string;
         }
 
         $inquote   = false;
-        $quoting   = false;
+        $escaped   = false;
         $incomment = 0;
         $newstring = '';
 
         for ($n = 0; $n < strlen($string); $n++) {
-            if ($quoting) {
+            if ($escaped) {
                 if ($incomment == 0) {
                     $newstring .= $string[$n];
                 } else if ($comment !== null) {
                     $comment .= $string[$n];
                 }
-                $quoting = false;
+                $escaped = false;
             } else if ($string[$n] == '\\') {
-                $quoting = true;
+                $escaped = true;
             } else if (!$inquote && $incomment > 0 && $string[$n] == ')') {
                 $incomment--;
                 if ($incomment == 0 && $comment !== null) {
@@ -228,15 +302,13 @@ class MIME_Type
 
     /**
      * Get a MIME type's media
-     *
-     * @note 'media' refers to the portion before the first slash
+     * Note: 'media' refers to the portion before the first slash
      *
      * @param string $type MIME type to get media of
      *
      * @return string $type's media
-     * @static
      */
-    function getMedia($type)
+    public static function getMedia($type)
     {
         $tmp = explode('/', $type);
         return strtolower(trim(MIME_Type::stripComments($tmp[0], $null)));
@@ -249,9 +321,8 @@ class MIME_Type
      * @param string $type MIME type to get subtype of
      *
      * @return string $type's subtype, null if invalid mime type
-     * @static
      */
-    function getSubType($type)
+    public static function getSubType($type)
     {
         $tmp = explode('/', $type);
         if (!isset($tmp[1])) {
@@ -269,7 +340,7 @@ class MIME_Type
      *
      * @return string MIME type string
      */
-    function get()
+    public function get()
     {
         $type = strtolower($this->media . '/' . $this->subType);
         if (count($this->parameters)) {
@@ -284,18 +355,18 @@ class MIME_Type
     /**
      * Is this type experimental?
      *
-     * @note Experimental types are denoted by a leading 'x-' in the media or
+     * Note: Experimental types are denoted by a leading 'x-' in the media or
      *       subtype, e.g. text/x-vcard or x-world/x-vrml.
      *
      * @param string $type MIME type to check
      *
      * @return boolean true if $type is experimental, false otherwise
-     * @static
      */
-    function isExperimental($type)
+    public static function isExperimental($type)
     {
-        if (substr(MIME_Type::getMedia($type), 0, 2) == 'x-' ||
-            substr(MIME_Type::getSubType($type), 0, 2) == 'x-') {
+        if (substr(MIME_Type::getMedia($type), 0, 2) == 'x-'
+            || substr(MIME_Type::getSubType($type), 0, 2) == 'x-'
+        ) {
             return true;
         }
         return false;
@@ -305,14 +376,13 @@ class MIME_Type
     /**
      * Is this a vendor MIME type?
      *
-     * @note Vendor types are denoted with a leading 'vnd. in the subtype.
+     * Note: Vendor types are denoted with a leading 'vnd. in the subtype.
      *
      * @param string $type MIME type to check
      *
      * @return boolean true if $type is a vendor type, false otherwise
-     * @static
      */
-    function isVendor($type)
+    public static function isVendor($type)
     {
         if (substr(MIME_Type::getSubType($type), 0, 4) == 'vnd.') {
             return true;
@@ -327,9 +397,8 @@ class MIME_Type
      * @param string $type MIME type to check
      *
      * @return boolean true if $type is a wildcard, false otherwise
-     * @static
      */
-    function isWildcard($type)
+    public static function isWildcard($type)
     {
         if ($type == '*/*' || MIME_Type::getSubtype($type) == '*') {
             return true;
@@ -348,9 +417,8 @@ class MIME_Type
      * @param string $type MIME type to check
      *
      * @return boolean true if there was a match, false otherwise
-     * @static
      */
-    function wildcardMatch($card, $type)
+    public static function wildcardMatch($card, $type)
     {
         if (!MIME_Type::isWildcard($card)) {
             return false;
@@ -377,7 +445,7 @@ class MIME_Type
      *
      * @return void
      */
-    function addParameter($name, $value, $comment = false)
+    public function addParameter($name, $value, $comment = false)
     {
         $tmp = new MIME_Type_Parameter();
 
@@ -395,32 +463,28 @@ class MIME_Type
      *
      * @return void
      */
-    function removeParameter($name)
+    public function removeParameter($name)
     {
         unset($this->parameters[$name]);
     }
 
-
     /**
      * Autodetect a file's MIME-type
      *
-     * This function may be called staticly.
-     *
-     * @internal Tries to use fileinfo extension at first. If that
-     *  does not work, mime_magic is used. If this is also not available
-     *  or does not succeed, "file" command is tried to be executed with
-     *  System_Command. When that fails, too, then we use our in-built
-     *  extension-to-mimetype-mapping list.
-     *
      * @param string $file   Path to the file to get the type of
      * @param bool   $params Append MIME parameters if true
      *
      * @return string $file's MIME-type on success, PEAR_Error otherwise
      *
-     * @since 1.0.0beta1
-     * @static
+     * @since 1.3.0
+     *
+     * @internal Tries to use fileinfo extension at first. If that
+     *  does not work, mime_magic is used. If this is also not available
+     *  or does not succeed, "file" command is tried to be executed with
+     *  System_Command. When that fails, too, then we use our in-built
+     *  extension-to-mimetype-mapping list.
      */
-    function autoDetect($file, $params = false)
+    protected function _autoDetect($file, $params = false)
     {
         // Sanity checks
         if (!file_exists($file)) {
@@ -431,33 +495,41 @@ class MIME_Type
             return PEAR::raiseError("File \"$file\" is not readable");
         }
 
-        if (function_exists('finfo_file')) {
-            $finfo = finfo_open(FILEINFO_MIME);
-            $type  = finfo_file($finfo, $file);
-            finfo_close($finfo);
-            if ($type !== false && $type !== '') {
-                return MIME_Type::_handleDetection($type, $params);
+        if ($this->useFinfo && function_exists('finfo_file')) {
+            $finfo = finfo_open(FILEINFO_MIME, $this->magicFile);
+            if ($finfo) {
+                $type  = finfo_file($finfo, $file);
+                finfo_close($finfo);
+                if ($type !== false && $type !== '') {
+                    return MIME_Type::_handleDetection($type, $params);
+                }
             }
         }
 
-        if (function_exists('mime_content_type')) {
+        if ($this->useMimeContentType && function_exists('mime_content_type')) {
             $type = mime_content_type($file);
             if ($type !== false && $type !== '') {
                 return MIME_Type::_handleDetection($type, $params);
             }
         }
 
-        @include_once 'System/Command.php';
-        if (class_exists('System_Command')) {
-            return MIME_Type::_handleDetection(
-                MIME_Type::_fileAutoDetect($file),
-                $params
-            );
+        if ($this->useFileCmd) {
+            @include_once 'System/Command.php';
+            if (class_exists('System_Command')) {
+                $type = $this->_fileAutoDetect($file);
+                if ($type !== false && $type !== '') {
+                    return MIME_Type::_handleDetection($type, $params);
+                }
+            }
         }
 
-        require_once 'MIME/Type/Extension.php';
-        $mte = new MIME_Type_Extension();
-        return $mte->getMIMEType($file);
+        if ($this->useExtension) {
+            include_once 'MIME/Type/Extension.php';
+            $mte = new MIME_Type_Extension();
+            return $mte->getMIMEType($file);
+        }
+
+        return PEAR::raiseError("Sorry, couldn't determine file type.");
     }
 
 
@@ -469,7 +541,7 @@ class MIME_Type
      *
      * @return string $file's MIME-type on success, PEAR_Error otherwise
      */
-    function _handleDetection($type, $params)
+    protected static function _handleDetection($type, $params)
     {
         // _fileAutoDetect() may have returned an error.
         if (PEAR::isError($type)) {
@@ -493,18 +565,16 @@ class MIME_Type
     /**
      * Autodetect a file's MIME-type with 'file' and System_Command
      *
-     * This function may be called staticly.
-     *
      * @param string $file Path to the file to get the type of
      *
      * @return string $file's MIME-type
      *
      * @since 1.0.0beta1
-     * @static
      */
-    function _fileAutoDetect($file)
+    protected function _fileAutoDetect($file)
     {
-        $cmd = new System_Command();
+        $cmd   = new System_Command();
+        $magic = '';
 
         // Make sure we have the 'file' command.
         $fileCmd = PEAR::getStaticProperty('MIME_Type', 'fileCmd');
@@ -512,12 +582,15 @@ class MIME_Type
             unset($cmd);
             return PEAR::raiseError("Can't find file command \"{$fileCmd}\"");
         }
+        if (strlen($this->magicFile)) {
+            $magic = '--magic-file ' . escapeshellarg($this->magicFile);
+        }
 
-        $cmd->pushCommand($fileCmd, "-bi " . escapeshellarg($file));
+        $cmd->pushCommand($fileCmd, $magic, "-bi " . escapeshellarg($file));
         $res = $cmd->execute();
         unset($cmd);
 
         return $res;
     }
-
-}
\ No newline at end of file
+}
+?>
