@@ -3,17 +3,15 @@ if (!defined('PHPUnit_MAIN_METHOD')) {
     define('PHPUnit_MAIN_METHOD', 'MIME_Type_AllTests::main');
 }
 
-require_once 'PHPUnit/TextUI/TestRunner.php';
-
-chdir(dirname(__FILE__) . '/../');
-require_once 'TypeTest.php';
-require_once 'MIME_Type_ExtensionTest.php';
-
+require_once 'PHPUnit/Autoload.php';
 
 class MIME_Type_AllTests
 {
     public static function main()
     {
+        //PEAR's phpunit call did not load it automatically
+        require_once __DIR__ . '/bootstrap.php';
+
         PHPUnit_TextUI_TestRunner::run(self::suite());
     }
 
@@ -21,8 +19,9 @@ class MIME_Type_AllTests
     {
         $suite = new PHPUnit_Framework_TestSuite('MIME_Type tests');
         /** Add testsuites, if there is. */
-        $suite->addTestSuite('MIME_TypeTest');
-        $suite->addTestSuite('MIME_Type_ExtensionTest');
+        $suite->addTestFiles(
+            glob(__DIR__ . '/*Test.php', GLOB_BRACE)
+        );
 
         return $suite;
     }
@@ -31,4 +30,4 @@ class MIME_Type_AllTests
 if (PHPUnit_MAIN_METHOD == 'MIME_Type_AllTests::main') {
     MIME_Type_AllTests::main();
 }
-?>
+?>
\ No newline at end of file
